﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ABC.SSRS
{
    public class SafeDivide
    {
        /// <summary>
        /// This is to provide safe dividing of two numbers in reporting services onumer / odenom
        /// </summary>
        /// <param name="onumer"></param>
        /// <param name="odenom"></param>
        /// <returns></returns>

        public static decimal? Divide(object onumer, object odenom)
        {

            if (onumer == null && odenom == null) return null;
            if (onumer == null) return 0;
            if (odenom == null) return 0;

            decimal inumer;
            decimal idenom;
            try
            {
                inumer = CastAsDecimal(onumer);
                idenom = CastAsDecimal(odenom);
            }
            catch
            {
                return 0;
            }
            if (idenom == 0) return 0;
            try
            {
                return decimal.Divide(inumer, idenom);
            }
            catch
            {
                return 0;
            }
        }

        /// <summary>
        /// This is to provide safe dividing of two numbers in reporting services onumer / odenom
        /// </summary>
        /// <param name="onumer"></param>
        /// <param name="odenom"></param>
        /// <returns></returns>
        public static decimal? Variance(object onumer, object odenom)
        {

            if (onumer == null && odenom == null) return null;
            if (onumer == null) return 0;
            if (odenom == null) return 0;

            decimal inumer;
            decimal idenom;
            try
            {
                inumer = CastAsDecimal(onumer);
                idenom = Math.Abs(CastAsDecimal(odenom));
            }
            catch
            {
                return 0;
            }
            if (idenom == 0) return 0;
            try
            {
                return decimal.Divide(inumer, idenom);
            }
            catch
            {
                return 0;
            }
        }


        /// <summary>
        /// This is to provide difference between safe dividing of two numbers in reporting services (onumer1 / odenom1) - (onumer2 / odenom2)
        /// </summary>
        /// <param name="onumer1"></param>
        /// <param name="odenom1"></param>
        /// <param name="onumer2"></param>
        /// <param name="odenom2"></param>
        /// <returns></returns>
        /// 
        public static decimal? DivideChange(object onumer1, object odenom1, object onumer2, object odenom2)
        {

            if (onumer1 == null && odenom1 == null && onumer2 == null && odenom2 == null) return null;

            decimal? inumer1 = Divide(onumer1, odenom1);
            decimal? inumer2 = Divide(onumer2, odenom2);

            try
            {
                if ((inumer1 == null || inumer1 == 0) && !(inumer2 == null || inumer2 == 0)) return inumer2 * -1;
                if ((inumer2 == null || inumer2 == 0) && !(inumer1 == null || inumer1 == 0)) return inumer1;
                return inumer1 - inumer2;
            }
            catch
            {
                return 0;
            }
        }



        /// <summary>
        /// This is to provide difference between safe dividing of two numbers in reporting services (onumer1 / odenom1) - (onumer2 / odenom2)
        /// </summary>
        /// <param name="onumer1"></param>
        /// <param name="odenom1"></param>
        /// <param name="onumer2"></param>
        /// <param name="odenom2"></param>
        /// <param name="onumer3"></param>
        /// <param name="odenom3"></param>
        /// <param name="onumer4"></param>
        /// <param name="odenom4"></param>
        /// <returns></returns>
        /// 
        public static decimal? MixEffect(object onumer1, object odenom1, object onumer2, object odenom2, object onumer3, object odenom3, object onumer4, object odenom4)
        {

            if (onumer1 == null && odenom1 == null && onumer2 == null && odenom2 == null && onumer3 == null && odenom3 == null && onumer4 == null && odenom4 == null) return null;

            decimal? inumer1 = DivideChange(onumer1, odenom1, onumer2, odenom2) * Divide(onumer3, odenom3);
            decimal? inumer2 = DivideChange(onumer3, odenom3, onumer4, odenom4) * Divide(onumer1, odenom1);

            try
            {
                if ((inumer1 == null || inumer1 == 0) && !(inumer2 == null || inumer2 == 0)) return inumer2;
                if ((inumer2 == null || inumer2 == 0) && !(inumer1 == null || inumer1 == 0)) return inumer1;
                return inumer1 + inumer2;
            }
            catch
            {
                return 0;
            }
        }


        private static decimal CastAsDecimal(object _obj)
        {
            switch (_obj.GetType().Name)
            {
                case "Int32":
                    return decimal.Parse(((Int32)_obj).ToString());
                case "Double":
                    return decimal.Parse(((double)_obj).ToString());
                case "Decimal":
                    return (decimal)_obj;
                default:
                    return 0;
            }
        }
    }
}
